package java.security;

import sun.security.jca.JCAUtil;
import java.nio.ByteBuffer;

public abstract class MessageDigestSpi
{
    private byte[] tempArray;
    
    public MessageDigestSpi() {
    }
    
    protected int engineGetDigestLength() {
        return 0;
    }
    
    protected abstract void engineUpdate(byte p0);
    
    protected abstract void engineUpdate(byte[] p0, int p1, int p2);
    
    protected void engineUpdate(ByteBuffer byteBuffer) {
        if (!byteBuffer.hasRemaining()) {
            return;
        }
        if (byteBuffer.hasArray()) {
            byte[] array = byteBuffer.array();
            int arrayOffset = byteBuffer.arrayOffset();
            int position = byteBuffer.position();
            int limit = byteBuffer.limit();
            this.engineUpdate(array, arrayOffset + position, limit - position);
            byteBuffer.position(limit);
        }
        else {
            int i = byteBuffer.remaining();
            int tempArraySize = JCAUtil.getTempArraySize(i);
            if (this.tempArray == null || tempArraySize > this.tempArray.length) {
                this.tempArray = new byte[tempArraySize];
            }
            while (i > 0) {
                int min = Math.min(i, this.tempArray.length);
                byteBuffer.get(this.tempArray, 0, min);
                this.engineUpdate(this.tempArray, 0, min);
                i -= min;
            }
        }
        return;
    }
    
    protected abstract byte[] engineDigest();
    
    protected int engineDigest(byte[] array, int n, int n2) throws DigestException {
        byte[] engineDigest = this.engineDigest();
        if (n2 < engineDigest.length) {
            throw new DigestException("partial digests not returned");
        }
        if (array.length - n < engineDigest.length) {
            throw new DigestException("insufficient space in the output buffer to store the digest");
        }
        System.arraycopy(engineDigest, 0, array, n, engineDigest.length);
        return engineDigest.length;
    }
    
    protected abstract void engineReset();
    
    public Object clone() throws CloneNotSupportedException {
        if ((this instanceof Cloneable)) {
            return super.clone();
        }
        throw new CloneNotSupportedException();
    }
}
